#ifndef GRAM_EXECUTE_H
#define GRAM_EXECUTE_H

#include "kv.h"
typedef struct exe_s exe_t;
typedef struct exe_fd_s exe_fd_t;

#include "parse_common.h"

typedef enum {
	PROC_START,
	PROC_STOP,      /* stop gracefully (UNIX: SIGTERM) */
	PROC_KILL,      /* kill (UNIX: SIGKILL) */
	PROC_PAUSE,
	PROC_RESUME
} proc_st_change_t;

typedef struct exe_proc_st_s {
	char *id;                 /* all, if NULL */
	proc_st_change_t newst;   /* 1 for start, 0 for stop */
} exe_proc_st_t;

struct exe_fd_s {
	char *proc;
	int fd;
};

typedef struct exe_reg_pipe_s {
	exe_fd_t *from;
	exe_fd_t *to;
} exe_reg_pipe_t;

typedef struct exe_create_proc_s {
	char *id;
	int virt;
	char *cmd;
	kvlist_t *args;
	kvlist_t *props;

	/* store the pointer to the newly created proc for a proc_start() */
	proc_virt_t *proc;
} exe_create_proc_t;

typedef struct exe_query_s {
	char *q;
	char *obj;
	char *cookie;
} exe_query_t;

/* === */

typedef enum {
	EXE_DUMP_INTERNALS,
	EXE_SUBTX,
	EXE_QUERY,
	EXE_EXIT,
	EXE_PROC_ST,
	EXE_REG_PIPE,
	EXE_CREATE_PROC,
} exe_op_t;


struct exe_s {
	exe_op_t op;
	union {
		exe_proc_st_t      proc_st;
		exe_reg_pipe_t     reg_pipe;
		exe_create_proc_t  create_proc;
		char               *dump_internals_name;
		transaction_t      *subtx;
		exe_query_t        query;
	} data;
	exe_t *next;
};


exe_fd_t *exe_fdref_(char *id, int fd);   /* strclone() id before calling */
exe_fd_t *exe_fdref(char *id, char *fd);  /* strclone() id before calling */
exe_fd_t *exe_fdclone(exe_fd_t *f);
void exe_free_fd(exe_fd_t *f);

void exe_append_create_proc(parser_t *p, char *id, int virt, char *cmd, kvlist_t *args, kvlist_t *props);
void exe_append_reg_pipe(parser_t *p, exe_fd_t *from, exe_fd_t *to);
void exe_append_proc_st(parser_t *p, char *id, proc_st_change_t newst);
void exe_append_exit(parser_t *p);
void exe_append_dump_internals(parser_t *p, char *name);


/* execute and remove the script from the current transaction, leaving an
   active but empty transaction for the parser */
void exe_flush(parser_t *p);


/* increase reference counter of tx if not NULL or create new tx with ref counter 1 */
transaction_t *tx_this_or_new(transaction_t *tx, const char *name, fd_t *query_out);

/* eof without flush means discard probably partial and borken script in some
   situations ([cmd]) */
void tx_discard(transaction_t *tx);

/* decrease reference counter of tx by 1 and execute the script if it's 0;
   return tx or NULL if script has been executed. Also frees the script after
   execution */
transaction_t *tx_finish(transaction_t *tx);

/* The following two calls are the two parts of tx_finish() and must be
   called in this order, only once for a tx. For [new] it is useful to
   have the second call only from start() while do the first call
   from init(). This way everything is created from init() in stopped
   mode and will be started from start().

   Warning: transaction on which these calls can be used must not be
   derived from any existing transaction (so that ->refcnt is always 1).
*/

/* create all processes in a tx, but do not start them yet */
void tx_finish_create(transaction_t *tx);

/* run all processes and close tx, after a tx_finish_create() */
void tx_finish_run(transaction_t *tx);

#endif
