/*  plumb - the pipe shell - execute plumb scripts on the fly
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "fd.h"
#include "plumb.h"
#include "virt.h"

typedef struct {
	parser_t *parser;
} cmd_t;

static virt_ctrl_t cmd_input(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	cmd_t *c = proc->data;
	plumb_parse_stream_buffer(c->parser, data, len);
	return VC_OK;
}

static void cmd_free(proc_virt_t *proc)
{
	cmd_t *c = proc->data;
	plumb_parse_stream_close(c->parser);
	free(c);
}

int vp_cmd_init(proc_virt_t *p, kvlist_t *args)
{
	cmd_t *c;
	char *inp_name;
	int len;

	vp_stdfds_set(p, VP_REQ_QUIT_ON_UNBIND, VP_REQ_ANY, VP_REQ_ANY, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	c = malloc(sizeof(parser_t));

	len = strlen(p->h.id);
	inp_name = malloc(len + 8);
	memcpy(inp_name, "[cmd]:", 6);
	memcpy(inp_name+6, p->h.id, len+1);
	c->parser = plumb_parse_stream_open(NULL, NULL, inp_name, fd_ref_(p, vstdout));

	/* allocate a process descriptor */
	p->input  = cmd_input;
	p->timer  = NULL;
	p->start  = NULL;
	p->free   = cmd_free;
	p->data   = c;

	return 0;
}
