/*  plumb - the pipe shell - watch for signals from background real processes (RPs)
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "vp.h"
#include "plumb.h"

/* This slightly special virtual process is reading the libporty control socket
   of real processes to detect when they quit, in order to:
   - trigger sticky-check and allow plumb to exit cleanly when all sticky RPs died
   - emit events through the supvervisor
*/

proc_virt_t *procwatch = NULL;

static virt_ctrl_t procwatch_input(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	proc_real_t *died = NULL;

/*	fprintf(stderr, "PROCWATCH read: %s %d\n", data, len); */


	died = proc->h.fds[local_fd].pipe->from->proc.r;

	if (len > 0) {
		char buff[512], *id;
		int plen, maxlen, mlen, olen;

		if (strlen(died->h.id) > sizeof(buff) - 128)
			id = "<id_too_long>";
		else
			id = died->h.id;
		plen = sprintf(buff, "plumb_procwatch %s ", id);
		maxlen = sizeof(buff) - plen - 2;
		mlen = (len > maxlen) ? maxlen : len;
		olen = mlen+plen;

		memcpy(buff+plen, data, mlen);
		fd_write(&proc->h.fds[1], buff, olen);
	}

	/* mark the process dead only if control socket sent an eof or wrote one of the die-messages */
	if (len != 0)
		if (strncmp(data, "END", 3) != 0)
			return VC_OK;


	if ((died->h.sticky) && (died->h.state != PS_STOPPED))
		plumb_sticky_died();

	died->h.state = PS_STOPPED;

	return VC_OK;
}

static virt_ctrl_t procwatch_sink_eof(proc_virt_t *proc, int local_fd)
{
	return procwatch_input(proc, local_fd, NULL, 0);
}

static virt_ctrl_t procwatch_source_eof(proc_virt_t *proc, int local_fd)
{
	if (!plumb_shutdown) {
		plumb_error("Source eof on procwatch: can't emit events anymore");
		return VC_QUIT;
	}
	return VC_OK;
}


int vp_procwatch_init(proc_virt_t *p, kvlist_t *args)
{
	/* allocate a process descriptor */
	p->input = procwatch_input;
	p->sink_eof = procwatch_sink_eof;
	p->source_eof = procwatch_source_eof;

	procwatch = p;

	return 0;
}
